@@ -2,7 +2,7 @@ package Tibco::Rv::Transport;
 
 
 use vars qw/ $VERSION $PROCESS /;
-$VERSION = '1.02';
+$VERSION = '1.10';
 
 
 use constant PROCESS_TRANSPORT => 10;
@@ -14,8 +14,8 @@ use constant TIMER_BATCH => 1;
 my ( %defaults );
 BEGIN
 {
-   %defaults = ( service => undef, network => undef, daemon => 'tcp:7500',
-      batchMode => DEFAULT_BATCH, description => '' );
+   %defaults = ( service => '', network => '', daemon => 'tcp:7500',
+      batchMode => DEFAULT_BATCH, description => undef );
 }
 
 
@@ -29,16 +29,17 @@ sub new
    my ( $class ) = ref( $proto ) || $proto;
    my ( $self ) = $class->_new;
 
-   @$self{ qw/ service network daemon / } =
-      @params{ qw/ service network daemon / };
+   my ( @snd ) = qw/ service network daemon /;
+   map { $params{$_} = '' unless ( defined $params{$_} ) } @snd;
+   @$self{ @snd } = @params{ @snd };
 
-   my ( $status ) =
-      Tibco::Rv::Transport_Create( @$self{ qw/ id service network daemon / } );
+   my ( $status ) = Tibco::Rv::Transport_Create( @$self{ 'id', @snd } );
    Tibco::Rv::die( $status ) unless ( $status == Tibco::Rv::OK );
 
    $self->batchMode( $params{batchMode} )
       if ( $params{batchMode} != DEFAULT_BATCH );
-   $self->description( $params{description} ) if ( $params{description} ne '' );
+   $self->description( $params{description} )
+      if ( defined $params{description} );
 
    return $self;
 }
@@ -110,6 +111,7 @@ sub description
 sub _setDescription
 {
    my ( $self, $description ) = @_;
+   $description = '' unless ( defined $description );
    my ( $status ) =
       Tibco::Rv::tibrvTransport_SetDescription( $self->{id}, $description );
    Tibco::Rv::die( $status ) unless ( $status == Tibco::Rv::OK );
@@ -194,12 +196,12 @@ daemon, which routes messages to other Tibco programs.
       description => $description,
       batchMode => $batchMode
 
-Creates a C<Tibco::Rv::Transport>.  If not specified, service defaults to
-the rendezvous service, network defaults to no network, and daemon defaults
-to 'tcp:7500' (see your TIB/Rendezvous documentation for discussion on the
-service/network/daemon parameters).  Description defaults to '', and batchMode
-defaults to Tibco::Rv::Transport::DEFAULT_BATCH.  If Tibco::Rv was built
-against an Rv 6.x version, then the constructor will die with a
+Creates a C<Tibco::Rv::Transport>.  If not specified, service defaults to ''
+(the rendezvous service), network defaults to '' (no network), and daemon
+defaults to 'tcp:7500' (see your TIB/Rendezvous documentation for discussion
+on the service/network/daemon parameters).  Description defaults to C<undef>,
+and batchMode defaults to Tibco::Rv::Transport::DEFAULT_BATCH.  If Tibco::Rv
+was built against an Rv 6.x version, then the constructor will die with a
 Tibco::Rv::VERSION_MISMATCH Status message if you attempt to set batchMode
 to anything other than Tibco::Rv::Transport::DEFAULT_BATCH.
 
@@ -231,6 +233,11 @@ Sets the description of C<$transport>.  Description identifies this transport
 to TIB/Rendezvous components.  It is displayed in the browser administration
 interface.
 
+Although description defaults to C<undef>, if you try to set it to C<undef>,
+it ends up being '' (this matches the behaviour of the C API, if you consider
+Perl C<undef> to be equivalent to C NULL, and Perl '' to be equivalent to C
+"").
+
 =item $batchMode = $transport->batchMode
 
 Returns the batchMode of C<$transport>.  If Tibco::Rv was built against
